@@ -4,7 +4,7 @@ use warnings;
 
 package Devel::Isa::Explainer;
 
-our $VERSION = '0.001001';
+our $VERSION = '0.002000';
 
 # ABSTRACT: Pretty Print Hierarchies of Subs in Packages
 
@@ -16,18 +16,15 @@ use Carp           ('croak');
 use Package::Stash ();
 use MRO::Compat    ();
 
-BEGIN { *import = \&Exporter::import }    ## no critic (ProhibitCallsToUnexportedSubs)
-
-our @EXPORT_OK = qw( explain_isa );
-
 # Perl critic is broken. This is not a void context.
 ## no critic (BuiltinFunctions::ProhibitVoidMap)
 use constant 1.03 ( { map { ( ( sprintf '_E%x', $_ ), ( sprintf ' (id: %s#%d)', __PACKAGE__, $_ ), ) } 1 .. 5 } );
 
-{
-  no strict 'refs';                       # namespace clean
-  delete ${ __PACKAGE__ . q[::] }{ sprintf '_E%x', $_ } for 1 .. 5;
-}
+use namespace::clean;
+
+BEGIN { *import = \&Exporter::import }    ## no critic (ProhibitCallsToUnexportedSubs)
+
+our @EXPORT_OK = qw( explain_isa );
 
 # These exist for twiddling, but are presently undocumented as their interface
 # is not deemed even remotely stable. Use at own risk.
@@ -64,8 +61,6 @@ sub explain_isa {
 }
 
 # -- no user servicable parts --
-sub _class_subs { return Package::Stash->new( $_[0] )->list_all_symbols('CODE') }
-
 sub _sub_type {
   my ($sub) = @_;
   return 'PRIVATE'   if $sub =~ /\A_/sx;
@@ -184,9 +179,10 @@ sub _pp_subs {
     my @subs = @{$cluster};
     while (@subs) {
       my $line = $INDENT;
-      while ( @subs and length $line < $MAX_WIDTH ) {
+    flowcontrol: {
         my $sub = shift @subs;
         $line .= $sub . q[, ];
+        redo flowcontrol if @subs and length $line < $MAX_WIDTH;
       }
       $cluster_out .= "$line\n";
     }
@@ -205,7 +201,7 @@ sub _pp_class {
   my $mro_order = _extract_mro($class);
   for my $mro_entry ( @{$mro_order} ) {
     $out .= colored( ['green'], $mro_entry->{class} ) . q[:];
-    my (%subs) = %{ $mro_entry->{subs} };
+    my (%subs) = %{ $mro_entry->{subs} || {} };
     if ( not keys %subs ) {
       $out .= " ()\n";
       next;
@@ -219,58 +215,70 @@ sub _pp_class {
 }
 
 sub _extract_mro {
-  my ($class) = @_;
-  my (@mro_order);
+  my ($class)     = @_;
   my ($seen_subs) = {};
 
-  # Walk down finding shadowing
-  ## no critic (ProhibitCallstoUnexportedSubs)
-  for my $isa ( @{ mro::get_linear_isa($class) } ) {
-    my (@subs) = _class_subs($isa);
-    if ( not @subs ) {
-      push @mro_order,
-        {
-        class => $isa,
-        subs  => {},
-        };
-      next;
-    }
-    my %sub_map;
-    for my $sub (@subs) {
-      $sub_map{$sub} = {
-        shadowed  => 0,
-        shadowing => 0,
-      };
+  ## no critic (ProhibitCallsToUnexportedSubs)
+  my (@isa) = @{ mro::get_linear_isa($class) };
 
-      # The first incarnation of a sub shadows the rest.
-      if ( not exists $seen_subs->{$sub} ) {
-        $seen_subs->{$sub} = $isa;
-      }
+  # Discover all subs and compute full MRO every time a new sub-name
+  # is found
+  for my $isa (@isa) {
+    for my $sub ( Package::Stash->new($isa)->list_all_symbols('CODE') ) {
+      next if exists $seen_subs->{$sub};
 
-      # If we are shadowed, mark ourselves shadowed,
-      # and mark all children as shadowers
-      if ( $seen_subs->{$sub} ne $isa ) {
-        $sub_map{$sub}->{shadowed} = 1;
-        for my $child_class (@mro_order) {
-          next unless exists $child_class->{subs}->{$sub};
-          $child_class->{subs}->{$sub}->{shadowing} = 1;
+      # Compute the full sub->package MRO table bottom up
+
+      $seen_subs->{$sub} = [];
+      my $currently_visible;
+      for my $class ( reverse @isa ) {
+        my $coderef = $class->can($sub) or next;
+
+        # Record the frame where the first new instance is seen.
+        if ( not defined $currently_visible or $currently_visible != $coderef ) {
+          unshift @{ $seen_subs->{$sub} }, $class;    # note: we're working bottom-up
+          $currently_visible = $coderef;
+          next;
         }
       }
     }
-    push @mro_order,
-      {
-      class => $isa,
-      subs  => \%sub_map,
-      };
   }
 
+  my $class_data = {};
+
+  # Group "seen subs" into class oriented structures,
+  # and classify them.
+  for my $sub ( keys %{$seen_subs} ) {
+    my @classes = @{ $seen_subs->{$sub} };
+
+    for my $isa ( @{ $seen_subs->{$sub} } ) {
+
+      # mark all subs both shadowing and shadowed until proven otherwise
+      $class_data->{$isa}->{$sub} = { shadowed => 1, shadowing => 1 };
+    }
+
+    # mark top-most sub unshadowed
+    $class_data->{ $classes[0] }->{$sub}->{shadowed} = 0;
+
+    # mark bottom-most sub unshadowing
+    $class_data->{ $classes[-1] }->{$sub}->{shadowing} = 0;
+
+  }
+
+  # Order class structures by MRO order
+  my (@mro_order) = map { { class => $_, subs => $class_data->{$_} || {} } } @isa;
+
   if ( 1 > @mro_order or ( 1 >= @mro_order and 1 > keys %{ $mro_order[0]->{subs} } ) ) {
 
     # Huh, No inheritance, and no subs. K.
     my $module_path = $class;
     $module_path =~ s{ (::|') }{/}sgx;
-    if ( not exists $INC{ $module_path . '.pm' } ) {
-      croak "No module called $class loaded" . _E5;
+
+    # TODO: Maybe don't make this fatal and return data context to user instead?
+    # Undecided, and will have to come after more complex concerns.
+    # - kentnl, Apr 2016
+    if ( not $INC{ $module_path . '.pm' } ) {
+      croak "No module called $class successfully loaded" . _E5;
     }
   }
   return \@mro_order;
@@ -290,7 +298,7 @@ Devel::Isa::Explainer - Pretty Print Hierarchies of Subs in Packages
 
 =head1 VERSION
 
-version 0.001001
+version 0.002000
 
 =head1 SYNOPSIS
 
